#!/bin/bash
# © Copyright [2020] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (“Micro Focus”) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

# Log files
TMP_LOG_FILE="/tmp/docker_change_cisnet.log"
DATE_NOW=$(date +%s)
LOG_FILE="/var/opt/novell/log/cis/docker_change_cisnet_$DATE_NOW.log"
conf="/etc/opt/novell/cis/config"
subnet="172.18.0.0"

usage() {
    cat <<EOF
CIS docker subnet changing utility

Usage: docker_change_cisnet.sh <OPTIONS>

GENERAL OPTIONS (OPTIONS):
  -h, --help            Displays information about this utility.
  --subnet  <IP>        To change the default docker network address (172.16.0.0) to the specified network address.

This utility changes the cisnet docker subnet. By default, this utility runs with no options and changes the subnet to 172.18.0.0/16.
To change to a specific subnet, use the “--subnet <IP>” option to provide the network address of a new subnet.
EOF
}

printandExit() {
    if [[ $1 -ne 0 ]]; then
        echo -en "Docker subnet changing failed." | tee -a "$TMP_LOG_FILE"
        mv "$TMP_LOG_FILE" "$LOG_FILE"
        echo -e " Check $LOG_FILE"
        exit 1
    else
        echo -en "Docker subnet change completed successfully." | tee -a "$TMP_LOG_FILE"
        mv "$TMP_LOG_FILE" "$LOG_FILE"
        echo -e " Check $LOG_FILE"
        exit 0
    fi
}

stopOrStartServices() {
    # Starting or Stopping all CIS core services, If running
    declare -a cis_service_names=(
        "gateway" "mgmt"
        "collector" "policy"
        "repcollector" "aggregator"
        "metadata" "auth"
        "repaggregator" "data"
    )

    for name in "${cis_service_names[@]}"; do
        if [ $1 = "stop" ]; then
            echo -e "stopping oes-cis-$name.service." | tee -a "$TMP_LOG_FILE"
        else
            echo -e "starting oes-cis-$name.service." | tee -a "$TMP_LOG_FILE"
        fi
        systemctl "$1" oes-cis-"$name".service

        if [ $? -ne 0 ]; then
            echo -e "Unable to $1 oes-cis-$name.service." | tee -a "$TMP_LOG_FILE"
            printandExit 1
        fi
    done

    if systemctl cat oes-dashboard.service &>/dev/null; then
        if [ $1 = "stop" ]; then
            echo -e "stopping oes-dashboard.service." | tee -a "$TMP_LOG_FILE"
        else
            echo -e "starting oes-dashboard.service." | tee -a "$TMP_LOG_FILE"
        fi
        systemctl "$1" oes-dashboard.service

        if [ $? -ne 0 ]; then
            echo -e "Unable to $1 oes-dashboard.service." | tee -a "$TMP_LOG_FILE"
            printandExit 1
        fi
    fi
}

writeDockerNwToConf() {
    IP=$(docker network inspect -f "{{(index (.IPAM.Config) 0).Subnet}}" cisnet)
    if [ $? -ne 0 ]; then
        echo -e "Unable to get cisnet subnet." | tee -a "$TMP_LOG_FILE"
        printandExit 1
    fi

    gateway=$(docker network inspect -f "{{(index (.IPAM.Config) 0).Gateway}}" cisnet)
    if [ $? -ne 0 ]; then
        echo -e "Unable to get cisnet gateway." | tee -a "$TMP_LOG_FILE"
        printandExit 1
    fi

    subnet=${IP/\/*/}

    IFS="."
    set -- $subnet
    range=$((0x$(printf "%02x%02x%02x%02x\n" $1 $2 $3 $4)))
    set -- $gateway
    g=$4

    g=$g+1
    v=$((range | g))
    ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
    sed -i '/IP_DATA/ c\IP_DATA="'"$ip"'"' $conf

    g=$g+1
    v=$((range | g))
    ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
    sed -i '/IP_METADATA/ c\IP_METADATA="'"$ip"'"' $conf

    g=$g+1
    v=$((range | g))
    ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
    sed -i '/IP_POLICY/ c\IP_POLICY="'"$ip"'"' $conf

    g=$g+1
    v=$((range | g))
    ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
    sed -i '/IP_AUTH/ c\IP_AUTH="'"$ip"'"' $conf

    g=$g+1
    v=$((range | g))
    ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
    sed -i '/IP_MGMT/ c\IP_MGMT="'"$ip"'"' $conf

    g=$g+1
    v=$((range | g))
    ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
    sed -i '/IP_COLLECTOR/ c\IP_COLLECTOR="'"$ip"'"' $conf

    g=$g+1
    v=$((range | g))
    ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
    sed -i '/IP_AGGREGATOR/ c\IP_AGGREGATOR="'"$ip"'"' $conf

    g=$g+1
    v=$((range | g))
    ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
    sed -i '/IP_REPCOLLECTOR/ c\IP_REPCOLLECTOR="'"$ip"'"' $conf

    g=$g+1
    v=$((range | g))
    ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
    sed -i '/IP_REPAGGREGATOR/ c\IP_REPAGGREGATOR="'"$ip"'"' $conf

    g=$g+1
    v=$((range | g))
    ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
    sed -i '/IP_GATEWAY/ c\IP_GATEWAY="'"$ip"'"' $conf

    g=$g+1
    v=$((range | g))
    ip=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
    sed -i '/IP_DASHBOARD/ c\IP_DASHBOARD="'"$ip"'"' $conf

    echo -e "Wrote new docker IP addresses." | tee -a "$TMP_LOG_FILE"
}

addiptables() {
    iptables=/usr/sbin/iptables
    IP=$(docker network inspect -f "{{(index (.IPAM.Config) 0).Subnet}}" cisnet) 2>/dev/null
    if [ $? != 0 ] || [ $IP == "" ]; then
        echo "cisnet not found" | tee -a "$TMP_LOG_FILE"
        printandExit 1
    fi

    if ! $iptables -w -C FORWARD -s $IP -j ACCEPT 2>/dev/null; then
        if ! $iptables -w -I FORWARD -s $IP -j ACCEPT 2>/dev/null; then
            echo "oes-cis: Failed to update docker to other network rule in FORWARD chain" | tee -a "$TMP_LOG_FILE"
            printandExit 1
        fi
        if ! $iptables -w -I FORWARD -d $IP -j ACCEPT 2>/dev/null; then
            echo "oes-cis: Failed to update other network to docker rule in FORWARD chain" | tee -a "$TMP_LOG_FILE"
            printandExit 1
        fi
        sysctl -w net.ipv4.ip_forward=1
        if ! $iptables -w -t nat -I POSTROUTING -s $IP -j MASQUERADE; then
            echo "oes-cis: Failed to update rule in POSTROUTING chain" | tee -a "$TMP_LOG_FILE"
            printandExit 1
        fi
    fi
}

cleariptables() {
    iptables=/usr/sbin/iptables
    IP=$(docker network inspect -f "{{(index (.IPAM.Config) 0).Subnet}}" cisnet)
    if [ $? != 0 ] || [ $IP == "" ]; then
        echo "cisnet not found" | tee -a "$TMP_LOG_FILE"
        printandExit 1
    fi

    if $iptables -w -C FORWARD -s $IP -j ACCEPT 2>/dev/null; then
        if ! $iptables -w -D FORWARD -s $IP -j ACCEPT 2>/dev/null; then
            echo -e "oes-cis: Failed to clear docker to other network rule in FORWARD chain" | tee -a "$TMP_LOG_FILE"
            printandExit 1
        fi
        if ! $iptables -w -D FORWARD -d $IP -j ACCEPT 2>/dev/null; then
            echo -e "oes-cis: Failed to clear other network to docker rule in FORWARD chain" | tee -a "$TMP_LOG_FILE"
            printandExit 1
        fi
        if ! $iptables -w -t nat -D POSTROUTING -s $IP -j MASQUERADE; then
            echo -e "oes-cis: Failed to clear rule in POSTROUTING chain" | tee -a "$TMP_LOG_FILE"
            printandExit 1
        fi
    fi
}

changeSubnet() {
    subnet=$1
    oldifs=$IFS
    IFS="."
    set -- $subnet
    range=$((0x$(printf "%02x%02x%02x%02x\n" $1 $2 $3 $4)))
    g=$4
    g=$g+1
    v=$((range | g))
    IFS=$oldifs
    gateway=$(printf "%d.%d.%d.%d" $(((v >> 24) & 255)) $(((v >> 16) & 255)) $(((v >> 8) & 255)) $(((v) & 255)))
    docker network inspect -f "{{(index (.IPAM.Config) 0).Subnet}}" cisnet 2>/dev/null
    if [ $? -ne 0 ]; then
        docker network create --driver=bridge --subnet="$subnet/16" --gateway="$gateway" cisnet &>/dev/null
        if [ $? -ne 0 ]; then
            echo -e "Docker network create failed.\n" | tee -a "$TMP_LOG_FILE"
            printandExit 1
        fi
        systemctl restart docker
        if [ $? -ne 0 ]; then
            echo -e "Docker is not running.\n" | tee -a "$TMP_LOG_FILE"
            printandExit 1
        fi
    fi
    echo -e "Recreated cisnet docker network under $subnet subnet" | tee -a "$TMP_LOG_FILE"
}

reconfigureDockerNetwork() {
    if [ "$2" != "$1" ]; then
        #stop services, remove docker network and recreate docker network & start services
        stopOrStartServices "stop"
        if [ $? -ne 0 ]; then
            echo -e "Unable to stop cis services." | tee -a "$TMP_LOG_FILE"
            printandExit 1
        fi

        cleariptables
        if [ $? -ne 0 ]; then
            echo -e "Unable to remove docker gateway from iptables." | tee -a "$TMP_LOG_FILE"
            printandExit 1
        fi

        docker network rm cisnet &>/dev/null
        if [ $? -ne 0 ]; then
            echo -e "Unable to remove docker network." | tee -a "$TMP_LOG_FILE"
            printandExit 1
        fi
        if [ $? -eq 0 ]; then
            echo -e "Removed cisnet docker network" | tee -a "$TMP_LOG_FILE"
        fi

        changeSubnet "$1"
        if [ $? -ne 0 ]; then
            echo -e "Unable to change subnet from $2 to $1." | tee -a "$TMP_LOG_FILE"
            printandExit 1
        fi

        addiptables
        if [ $? -ne 0 ]; then
            echo -e "Unable to add docker gateway from iptables." | tee -a "$TMP_LOG_FILE"
            printandExit 1
        fi

        writeDockerNwToConf
        if [ $? -ne 0 ]; then
            echo -e "Unable to write docker network to configuration file." | tee -a "$TMP_LOG_FILE"
            printandExit 1
        fi

        stopOrStartServices "start"
        if [ $? -ne 0 ]; then
            echo -e "Unable to start services." | tee -a "$TMP_LOG_FILE"
            printandExit 1
        fi
        printandExit 0
    fi
}

# Entry Point
while [ $# -ne 0 ]; do
    case $1 in
    --help)
        usage
        exit 0
        ;;
    -h)
        usage
        exit 0
        ;;
    --subnet)
        if [ -n "$2" ]; then
            subnet=$2
            shift
        else
            echo "--subnet requires as an argument the network address to change cisnet to." | tee -a "$TMP_LOG_FILE"
            printandExit 1
        fi
        ;;
    -*)
        echo "Unknown option $1." | tee -a "$TMP_LOG_FILE"
        usage
        printandExit 1
        ;;
    *)
        usage
        printandExit 1
        ;;
    esac
    shift
done

IP=$(docker network inspect -f "{{(index (.IPAM.Config) 0).Subnet}}" cisnet)
if [ $? -ne 0 ]; then
    echo -e "Unable to get cisnet subnet." | tee -a "$TMP_LOG_FILE"
    printandExit 1
fi
current=${IP/\/*/}

read -p"The CIS docker subnet ($current) will change to $subnet. Do you want to continue (Y/N): " -n1 C
echo
if [ ${C:-"N"} = "Y" ] || [ ${C:-"n"} = "y" ]; then
    reconfigureDockerNetwork $subnet $current
fi
